//This program is licensed under the GPLv3; see license.txt for more info.
//Copyright 2007 Sprite_tm

#include "io.h"
#include <avr/io.h>
#include <util/delay.h>

//Pins
#define LCDRS	PB0 
#define LCDRW	PA0 
#define LCDE	PB1 
#define LCDD4	PA3
#define LCDD5	PA7
#define LCDD6	PA4
#define LCDD7	PA6
#define PWM_LED PA5 //oc1b
#define PWM_CON PB2 //oc0a

void initio(void) {
    //Init gpio
    DDRA=(1<<PWM_LED)|(1<<LCDD4)|(1<<LCDD5)|(1<<LCDD6)|(1<<LCDD7)|(1<<LCDRW);
    DDRB=(1<<PWM_CON)|(1<<LCDRS)|(1<<LCDE);
    //Contrast: oc0a
    //Backlight: oc1b
    TCCR0A=0x83;
    TCCR0B=1;
    TCCR1A=0x21;
    TCCR1B=0x9;
}

//Set contrast
void lcd_contrast(const unsigned char c) {
    OCR0A=c;
}

//Set backlight
void lcd_backlight(const unsigned char c) {
    OCR1B=c;
}


//Read one nibble from the LCD. Used in lcd.c to read bytes.
unsigned char lcdrnibble(const char iscommand) {
    unsigned char r=0;
    DDRA&=~((1<<LCDD4)|(1<<LCDD5)|(1<<LCDD6)|(1<<LCDD7));

    if (!iscommand) PORTB|=(1<<LCDRS); else PORTB&=~(1<<LCDRS);
    PORTA|=(1<<LCDRW);
    PORTB|=(1<<LCDE);
    _delay_us(200);
    if (PINA&(1<<LCDD7)) r|=8;
    if (PINA&(1<<LCDD6)) r|=4;
    if (PINA&(1<<LCDD5)) r|=2;
    if (PINA&(1<<LCDD4)) r|=1;
    PORTB&=~(1<<LCDE);
    PORTA&=~(1<<LCDRW);
    DDRA|=(1<<LCDD4)|(1<<LCDD5)|(1<<LCDD6)|(1<<LCDD7);
    return r;
}


//Write one nibble to the LCD. Used in lcd.c to write bytes.
void lcdwnibble(const char iscommand, const char data) {
    if (!iscommand) PORTB|=(1<<LCDRS); else PORTB&=~(1<<LCDRS);
    PORTA&=~((1<<LCDD4)|(1<<LCDD5)|(1<<LCDD6)|(1<<LCDD7));
    if (data&1) PORTA|=(1<<LCDD4);
    if (data&2) PORTA|=(1<<LCDD5);
    if (data&4) PORTA|=(1<<LCDD6);
    if (data&8) PORTA|=(1<<LCDD7);
    PORTB|=(1<<LCDE);
    _delay_us(200);
    PORTB&=~(1<<LCDE);
    _delay_us(50);
}
