#include "lcd.h"
#include "io.h"
#include <util/delay.h>

//Write a char to the lcd
static void lcdwrite(char iscommand, unsigned char data) {
    lcdwnibble(iscommand,(data>>4));
    lcdwnibble(iscommand,(data&0xf));
}

//Read a char from the LCD
static unsigned char lcdread(char iscommand) {
    unsigned char t;
    t=(lcdrnibble(iscommand)<<4);
    t|=lcdrnibble(iscommand);
    return t;
}

//Wait till the LCD is ready for another command.
static void lcdwait(void) {
    while (lcdread(1)&0x80);
}

//Send a character to the LCDs ddram or cgram.
void lcdchar(const unsigned char c) {
    lcdwait();
    lcdwrite(0,c);
}

//Send a command to the LCD
void lcdcmd(const unsigned char c) {
    lcdwait();
    lcdwrite(1,c);
}


//Initialize the 8x2 LCD
void initlcd(void) {
    char t;
    _delay_ms(40);
    lcdwnibble(1,0x3);
    lcdwnibble(1,0x3);
    lcdwnibble(1,0x3);
    lcdwnibble(1,0x2);
    lcdwait();
    lcdcmd(0x2c);
    lcdcmd(0xC);
    lcdcmd(0x1);
    lcdcmd(0x6);

    lcdcmd(0x80);
}
